//
//  Copyright (C) 2004-2021 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// This implementation file provides some useful utility functions and classes.

#pragma once

/////////////////////////////////////////////////////////////////
/// \cond INTERNAL
/// Get and Set Layer and Group Order in current group in Display Manager.
class MG_PLATFORMBASE_API MgLayerGroupOrder : public MgNamedSerializable
{
    MG_DECL_DYNCREATE()
    DECLARE_CLASSNAME(MgLayerGroupOrder)

EXTERNAL_API:
    MgLayerGroupOrder() {}

    virtual ~MgLayerGroupOrder() {}

PUBLISHED_API:
    //////////////////////////////////////////////////////////////////
    /// Set the order
    //
    void SetDisplayItemOrder(INT32 order)
    {
        m_orderInGroup = order;
    }

    //////////////////////////////////////////////////////////////////
    /// Get the order
    //
    INT32 GetDisplayItemOrder()
    {
        return m_orderInGroup;
    }

INTERNAL_API:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Get the name of the object.
    ///
    /// \return
    /// Returns the name of object
    ///
    virtual STRING GetName() { return L""; }

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Does it allow to set the name
    ///
    /// \return
    /// true - allows to set the name
    /// false - can not set the name
    ///
    virtual bool CanSetName() { return false; }

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Serialize data to a stream
    ///
    /// \param stream
    /// Stream
    ///
    virtual void Serialize(MgStream* stream) {}

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Deserialize data from a stream
    ///
    /// \param stream
    /// Stream
    ///
    virtual void Deserialize(MgStream* stream) {}

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    /// \return
    /// Returns nothing
    ///
    virtual void Dispose() {}

private:
    INT32 m_orderInGroup{ -1 };  // Order in Group in Display Manager
};
/// \endcond
